import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/spaces_management/add_device_type/bloc/add_device_model_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/add_device_type/widgets/scrollable_grid_view_widget.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/subspace_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/tag.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag/views/assign_tag_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/tag_model/widgets/action_button_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';


class AddDeviceTypeWidget extends StatelessWidget {
  final List<ProductModel>? products;
  final ValueChanged<List<SelectedProduct>>? onProductsSelected;
  final List<SelectedProduct>? initialSelectedProducts;
  final List<SubspaceModel>? subspaces;
  final List<Tag>? spaceTags;
  final List<String>? allTags;
  final String spaceName;
  final Function(List<Tag>,List<SubspaceModel>?)? onSave;


  const AddDeviceTypeWidget(
      {super.key,
      this.products,
      this.initialSelectedProducts,
      this.onProductsSelected,
      this.subspaces,
      this.allTags,
      this.spaceTags,
      this.onSave,
      required this.spaceName});

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    final crossAxisCount = size.width > 1200
        ? 8
        : size.width > 800
            ? 5
            : 3;

    return BlocProvider(
        create: (_) => AddDeviceTypeBloc(initialSelectedProducts ?? []),
        child: Builder(
          builder: (context) => AlertDialog(
            title: const Text('Add Devices'),
            backgroundColor: ColorsManager.whiteColors,
            content: SingleChildScrollView(
              child: Container(
                width: size.width * 0.9,
                height: size.height * 0.65,
                color: ColorsManager.textFieldGreyColor,
                child: Column(
                  children: [
                    const SizedBox(height: 16),
                    Expanded(
                      child: Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 20.0),
                        child: ScrollableGridViewWidget(
                            products: products, crossAxisCount: crossAxisCount),
                      ),
                    ),
                  ],
                ),
              ),
            ),
            actions: [
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  CancelButton(
                    label: 'Cancel',
                    onPressed: () async {
                      Navigator.of(context).pop();
                    },
                  ),
                  ActionButton(
                    label: 'Continue',
                    backgroundColor: ColorsManager.secondaryColor,
                    foregroundColor: ColorsManager.whiteColors,
                    onPressed: () async {
                      final currentState =
                          context.read<AddDeviceTypeBloc>().state;
                      Navigator.of(context).pop();

                      if (currentState.isNotEmpty) {
                        final initialTags = generateInitialTags(
                          spaceTags: spaceTags,
                          subspaces: subspaces,
                        );

                        final dialogTitle = initialTags.isNotEmpty
                            ? 'Edit Device'
                            : 'Assign Tags';
                         await showDialog<bool>(
                          barrierDismissible: false,
                          context: context,
                          builder: (context) => AssignTagDialog(
                            products: products,
                            subspaces: subspaces,
                            addedProducts: currentState,
                            allTags: allTags,
                            spaceName: spaceName,
                            initialTags: initialTags,
                            title: dialogTitle,
                            onSave: (tags,subspaces){
                              onSave!(tags,subspaces);
                            },
                          ),
                        );
                      }
                    },
                  ),
                ],
              ),
            ],
          ),
        ));
  }

  List<Tag> generateInitialTags({
    List<Tag>? spaceTags,
    List<SubspaceModel>? subspaces,
  }) {
    final List<Tag> initialTags = [];

    if (spaceTags != null) {
      initialTags.addAll(spaceTags);
    }

    if (subspaces != null) {
      for (var subspace in subspaces) {
        if (subspace.tags != null) {
          initialTags.addAll(
            subspace.tags!.map(
              (tag) => tag.copyWith(location: subspace.subspaceName),
            ),
          );
        }
      }
    }

    return initialTags;
  }
}
