import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/create_space_template_body_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_body_model.dart';
import 'package:uuid/uuid.dart';

class Tag {
  String? uuid;
  String? tag;
  final ProductModel? product;
  String internalId;
  String? location;

  Tag(
      {this.uuid,
      required this.tag,
      this.product,
      String? internalId,
      this.location})
      : internalId = internalId ?? const Uuid().v4();

  factory Tag.fromJson(Map<String, dynamic> json) {
    final String internalId = json['internalId'] ?? const Uuid().v4();

    return Tag(
      uuid: json['uuid'] ?? '',
      internalId: internalId,
      tag: json['tag'] ?? '',
      product: json['product'] != null
          ? ProductModel.fromMap(json['product'])
          : null,
    );
  }

  Tag copyWith({
    String? tag,
    ProductModel? product,
    String? location,
  }) {
    return Tag(
      tag: tag ?? this.tag,
      product: product ?? this.product,
      location: location ?? this.location,
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'uuid': uuid,
      'tag': tag,
      'product': product?.toMap(),
    };
  }
}

extension TagModelExtensions on Tag {
  TagBodyModel toTagBodyModel() {
    return TagBodyModel()
      ..uuid = uuid ?? ''
      ..tag = tag ?? ''
      ..productUuid = product?.uuid;
  }

    CreateTagBodyModel toCreateTagBodyModel() {
    return CreateTagBodyModel()
      ..tag = tag ?? ''
      ..productUuid = product?.uuid;
  }
}
