import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/add_device_type/views/add_device_type_widget.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/space_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/subspace_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/tag.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/widgets/dialogs/icon_selection_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace/views/create_subspace_model_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/link_space_model/view/link_space_model_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/constants/space_icon_const.dart';

class CreateSpaceDialog extends StatefulWidget {
  final Function(String, String, List<SelectedProduct> selectedProducts,
      SpaceTemplateModel? spaceModel, List<SubspaceModel>? subspaces, List<Tag>? tags) onCreateSpace;
  final List<ProductModel>? products;
  final String? name;
  final String? icon;
  final bool isEdit;
  final List<SelectedProduct> selectedProducts;
  final SpaceModel? parentSpace;
  final SpaceModel? editSpace;
  final List<SpaceTemplateModel>? spaceModels;
  final List<SubspaceModel>? subspaces;
  final List<Tag>? tags;

  const CreateSpaceDialog(
      {super.key,
      this.parentSpace,
      required this.onCreateSpace,
      this.products,
      this.name,
      this.icon,
      this.isEdit = false,
      this.editSpace,
      this.selectedProducts = const [],
      this.spaceModels,
      this.subspaces,
      this.tags});

  @override
  CreateSpaceDialogState createState() => CreateSpaceDialogState();
}

class CreateSpaceDialogState extends State<CreateSpaceDialog> {
  String selectedIcon = Assets.location;
  SpaceTemplateModel? selectedSpaceModel;
  String enteredName = '';
  List<SelectedProduct> selectedProducts = [];
  late TextEditingController nameController;
  bool isOkButtonEnabled = false;
  bool isNameFieldInvalid = false;
  bool isNameFieldExist = false;
  List<SubspaceModel>? subspaces;
  List<Tag>? tags;

  @override
  void initState() {
    super.initState();
    selectedIcon = widget.icon ?? Assets.location;
    nameController = TextEditingController(text: widget.name ?? '');
    selectedProducts =
        widget.selectedProducts.isNotEmpty ? widget.selectedProducts : [];
    isOkButtonEnabled =
        enteredName.isNotEmpty || nameController.text.isNotEmpty;
  }

  @override
  Widget build(BuildContext context) {
    final screenWidth = MediaQuery.of(context).size.width;
    return AlertDialog(
      title: widget.isEdit
          ? const Text('Edit Space')
          : const Text('Create New Space'),
      backgroundColor: ColorsManager.whiteColors,
      content: SizedBox(
        width: screenWidth * 0.5,
        child: SingleChildScrollView(
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Expanded(
                flex: 1,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  //  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    const SizedBox(height: 50),
                    Stack(
                      alignment: Alignment.center,
                      children: [
                        Container(
                          width: screenWidth * 0.1,
                          height: screenWidth * 0.1,
                          decoration: const BoxDecoration(
                            color: ColorsManager.boxColor,
                            shape: BoxShape.circle,
                          ),
                        ),
                        SvgPicture.asset(
                          selectedIcon,
                          width: screenWidth * 0.04,
                          height: screenWidth * 0.04,
                        ),
                        Positioned(
                          top: 20,
                          right: 20,
                          child: InkWell(
                            onTap: _showIconSelectionDialog,
                            child: Container(
                              width: 24,
                              height: 24,
                              decoration: const BoxDecoration(
                                color: Colors.white,
                                shape: BoxShape.circle,
                              ),
                              child: SvgPicture.asset(
                                Assets.iconEdit,
                                width: 16,
                                height: 16,
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ],
                ),
              ),
              const SizedBox(width: 20),
              Expanded(
                flex: 2,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    TextField(
                      controller: nameController,
                      onChanged: (value) {
                        enteredName = value.trim();
                        setState(() {
                          isNameFieldExist = false;
                          isOkButtonEnabled = false;
                          isNameFieldInvalid = value.isEmpty;

                          if (!isNameFieldInvalid) {
                            if (_isNameConflict(value)) {
                              isNameFieldExist = true;
                              isOkButtonEnabled = false;
                            } else {
                              isNameFieldExist = false;
                              isOkButtonEnabled = true;
                            }
                          }
                        });
                      },
                      style: const TextStyle(color: Colors.black),
                      decoration: InputDecoration(
                        hintText: 'Please enter the name',
                        hintStyle: const TextStyle(
                          fontSize: 14,
                          color: ColorsManager.lightGrayColor,
                          fontWeight: FontWeight.w400,
                        ),
                        filled: true,
                        fillColor: ColorsManager.boxColor,
                        enabledBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(10),
                          borderSide: BorderSide(
                            color: isNameFieldInvalid || isNameFieldExist
                                ? ColorsManager.red
                                : ColorsManager.boxColor,
                            width: 1.5,
                          ),
                        ),
                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(10),
                          borderSide: const BorderSide(
                            color: ColorsManager.boxColor,
                            width: 1.5,
                          ),
                        ),
                      ),
                    ),
                    if (isNameFieldInvalid)
                      Padding(
                        padding: const EdgeInsets.only(top: 8.0),
                        child: Text(
                          '*Space name should not be empty.',
                          style: Theme.of(context)
                              .textTheme
                              .bodySmall
                              ?.copyWith(color: ColorsManager.red),
                        ),
                      ),
                    if (isNameFieldExist)
                      Padding(
                        padding: const EdgeInsets.only(top: 8.0),
                        child: Text(
                          '*Name already exist',
                          style: Theme.of(context)
                              .textTheme
                              .bodySmall
                              ?.copyWith(color: ColorsManager.red),
                        ),
                      ),
                    const SizedBox(height: 10),
                    selectedSpaceModel == null
                        ? DefaultButton(
                            onPressed: () {
                              _showLinkSpaceModelDialog(context);
                            },
                            backgroundColor: ColorsManager.textFieldGreyColor,
                            foregroundColor: Colors.black,
                            borderColor: ColorsManager.neutralGray,
                            borderRadius: 16.0,
                            padding: 10.0, // Reduced padding for smaller size
                            child: Align(
                              alignment: Alignment.centerLeft,
                              child: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  Padding(
                                    padding: const EdgeInsets.only(left: 6.0),
                                    child: SvgPicture.asset(
                                      Assets.link,
                                      width: screenWidth *
                                          0.015, // Adjust icon size
                                      height: screenWidth * 0.015,
                                    ),
                                  ),
                                  const SizedBox(width: 3),
                                  Flexible(
                                    child: Text(
                                      'Link a space model',
                                      overflow: TextOverflow
                                          .ellipsis, // Prevent overflow
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          )
                        : Container(
                            width: screenWidth * 0.35,
                            padding: const EdgeInsets.symmetric(
                                vertical: 10.0, horizontal: 16.0),
                            decoration: BoxDecoration(
                              color: ColorsManager.boxColor,
                              borderRadius: BorderRadius.circular(10),
                            ),
                            child: Wrap(
                              spacing: 8.0,
                              runSpacing: 8.0,
                              children: [
                                Chip(
                                    label: Text(
                                      selectedSpaceModel?.modelName ?? '',
                                      style: const TextStyle(
                                          color: ColorsManager.spaceColor),
                                    ),
                                    backgroundColor: ColorsManager.whiteColors,
                                    shape: RoundedRectangleBorder(
                                      borderRadius: BorderRadius.circular(8),
                                      side: const BorderSide(
                                        color: ColorsManager.transparentColor,
                                        width: 0,
                                      ),
                                    ),
                                    deleteIcon: Container(
                                      width: 24,
                                      height: 24,
                                      decoration: BoxDecoration(
                                        shape: BoxShape.circle,
                                        border: Border.all(
                                          color: ColorsManager.lightGrayColor,
                                          width: 1.5,
                                        ),
                                      ),
                                      child: const Icon(
                                        Icons.close,
                                        size: 16,
                                        color: ColorsManager.lightGrayColor,
                                      ),
                                    ),
                                    onDeleted: () => setState(() {
                                          this.selectedSpaceModel = null;
                                        })),
                              ],
                            ),
                          ),
                    const SizedBox(height: 25),
                    const Row(
                      children: [
                        Expanded(
                          child: Divider(
                            color: ColorsManager.neutralGray,
                            thickness: 1.0,
                          ),
                        ),
                        Padding(
                          padding: EdgeInsets.symmetric(horizontal: 8.0),
                          child: Text(
                            'OR',
                            style: TextStyle(
                              color: Colors.black,
                              fontWeight: FontWeight.bold,
                            ),
                          ),
                        ),
                        Expanded(
                          child: Divider(
                            color: ColorsManager.neutralGray,
                            thickness: 1.0,
                          ),
                        ),
                      ],
                    ),
                    const SizedBox(height: 25),
                    subspaces == null
                        ? DefaultButton(
                            onPressed: () {
                              _showSubSpaceDialog(context, enteredName, [],
                                  false, widget.products, subspaces);
                            },
                            backgroundColor: ColorsManager.textFieldGreyColor,
                            foregroundColor: Colors.black,
                            borderColor: ColorsManager.neutralGray,
                            borderRadius: 16.0,
                            padding: 10.0, // Reduced padding for smaller size
                            child: Align(
                              alignment: Alignment.centerLeft,
                              child: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  Padding(
                                    padding: const EdgeInsets.only(left: 6.0),
                                    child: SvgPicture.asset(
                                      Assets.addIcon,
                                      width: screenWidth *
                                          0.015, // Adjust icon size
                                      height: screenWidth * 0.015,
                                    ),
                                  ),
                                  const SizedBox(width: 3),
                                  Flexible(
                                    child: Text(
                                      'Create sub space',
                                      overflow: TextOverflow
                                          .ellipsis, // Prevent overflow
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          )
                        : SizedBox(
                            width: screenWidth * 0.35,
                            child: Container(
                              padding: const EdgeInsets.all(8.0),
                              decoration: BoxDecoration(
                                color: ColorsManager.textFieldGreyColor,
                                borderRadius: BorderRadius.circular(15),
                                border: Border.all(
                                  color: ColorsManager.textFieldGreyColor,
                                  width: 3.0, // Border width
                                ),
                              ),
                              child: Wrap(
                                spacing: 8.0,
                                runSpacing: 8.0,
                                children: [
                                  if (subspaces != null)
                                    ...subspaces!.map(
                                      (subspace) => Chip(
                                        label: Text(
                                          subspace.subspaceName,
                                          style: const TextStyle(
                                              color: ColorsManager
                                                  .spaceColor), // Text color
                                        ),
                                        backgroundColor: ColorsManager
                                            .whiteColors, // Chip background color
                                        shape: RoundedRectangleBorder(
                                          borderRadius: BorderRadius.circular(
                                              16), // Rounded chip
                                          side: const BorderSide(
                                              color: ColorsManager
                                                  .spaceColor), // Border color
                                        ),
                                      ),
                                    ),
                                  GestureDetector(
                                    onTap: () async {
                                      _showSubSpaceDialog(
                                          context,
                                          enteredName,
                                          [],
                                          false,
                                          widget.products,
                                          subspaces);
                                    },
                                    child: Chip(
                                      label: const Text(
                                        'Edit',
                                        style: TextStyle(
                                            color: ColorsManager.spaceColor),
                                      ),
                                      backgroundColor:
                                          ColorsManager.whiteColors,
                                      shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(16),
                                        side: const BorderSide(
                                            color: ColorsManager.spaceColor),
                                      ),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ),
                    const SizedBox(height: 10),
                    (tags?.isNotEmpty == true ||
                            subspaces?.any((subspace) =>
                                    subspace.tags?.isNotEmpty == true) ==
                                true)
                        ? SizedBox(
                            width: screenWidth * 0.25,
                            child: Container(
                              padding: const EdgeInsets.all(8.0),
                              decoration: BoxDecoration(
                                color: ColorsManager.textFieldGreyColor,
                                borderRadius: BorderRadius.circular(15),
                                border: Border.all(
                                  color: ColorsManager.textFieldGreyColor,
                                  width: 3.0, // Border width
                                ),
                              ),
                              child: Wrap(
                                spacing: 8.0,
                                runSpacing: 8.0,
                                children: [
                                  // Combine tags from spaceModel and subspaces
                                  ..._groupTags([
                                    ...?tags,
                                    ...?subspaces?.expand(
                                        (subspace) => subspace.tags ?? [])
                                  ]).entries.map(
                                        (entry) => Chip(
                                          avatar: SizedBox(
                                            width: 24,
                                            height: 24,
                                            child: SvgPicture.asset(
                                              entry.key.icon ??
                                                  'assets/icons/gateway.svg',
                                              fit: BoxFit.contain,
                                            ),
                                          ),
                                          label: Text(
                                            'x${entry.value}', // Show count
                                            style: const TextStyle(
                                              color: ColorsManager.spaceColor,
                                            ),
                                          ),
                                          backgroundColor:
                                              ColorsManager.whiteColors,
                                          shape: RoundedRectangleBorder(
                                            borderRadius:
                                                BorderRadius.circular(16),
                                            side: const BorderSide(
                                              color: ColorsManager.spaceColor,
                                            ),
                                          ),
                                        ),
                                      ),
                                  GestureDetector(
                                    onTap: () async {
                                      _showTagCreateDialog(context, enteredName,
                                          widget.products);
                                      // Edit action
                                    },
                                    child: Chip(
                                      label: const Text(
                                        'Edit',
                                        style: TextStyle(
                                            color: ColorsManager.spaceColor),
                                      ),
                                      backgroundColor:
                                          ColorsManager.whiteColors,
                                      shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(16),
                                        side: const BorderSide(
                                            color: ColorsManager.spaceColor),
                                      ),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          )
                        : DefaultButton(
                            onPressed: () {
                              _showTagCreateDialog(
                                  context, enteredName, widget.products);
                            },
                            backgroundColor: ColorsManager.textFieldGreyColor,
                            foregroundColor: Colors.black,
                            borderColor: ColorsManager.neutralGray,
                            borderRadius: 16.0,
                            padding: 10.0, // Reduced padding for smaller size
                            child: Align(
                              alignment: Alignment.centerLeft,
                              child: Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  Padding(
                                    padding: const EdgeInsets.only(left: 6.0),
                                    child: SvgPicture.asset(
                                      Assets.addIcon,
                                      width: screenWidth *
                                          0.015, // Adjust icon size
                                      height: screenWidth * 0.015,
                                    ),
                                  ),
                                  const SizedBox(width: 3),
                                  Flexible(
                                    child: Text(
                                      'Add devices',
                                      overflow: TextOverflow
                                          .ellipsis, // Prevent overflow
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          )
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
      actions: [
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Expanded(
              child: CancelButton(
                label: 'Cancel',
                onPressed: () => Navigator.of(context).pop(),
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: DefaultButton(
                onPressed: () {
                  if (nameController.text.isEmpty) {
                    setState(() {
                      isNameFieldInvalid = true;
                    });
                    return;
                  } else {
                    String newName = enteredName.isNotEmpty
                        ? enteredName
                        : (widget.name ?? '');
                    if (newName.isNotEmpty) {
                      widget.onCreateSpace(newName, selectedIcon,
                          selectedProducts, selectedSpaceModel,subspaces,tags);
                      Navigator.of(context).pop();
                    }
                  }
                },
                borderRadius: 10,
                backgroundColor: isOkButtonEnabled
                    ? ColorsManager.secondaryColor
                    : ColorsManager.grayColor,
                foregroundColor: ColorsManager.whiteColors,
                child: const Text('OK'),
              ),
            ),
          ],
        ),
      ],
    );
  }

  void _showIconSelectionDialog() {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return IconSelectionDialog(
          spaceIconList: spaceIconList,
          onIconSelected: (String selectedIcon) {
            setState(() {
              this.selectedIcon = selectedIcon;
            });
          },
        );
      },
    );
  }

  bool _isNameConflict(String value) {
    return (widget.parentSpace?.children.any((child) => child.name == value) ??
            false) ||
        (widget.parentSpace?.name == value) ||
        (widget.editSpace?.parent?.name == value) ||
        (widget.editSpace?.children.any((child) => child.name == value) ??
            false);
  }

  void _showLinkSpaceModelDialog(BuildContext context) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return LinkSpaceModelDialog(
          spaceModels: widget.spaceModels ?? [],
          onSave: (selectedModel) {
            if (selectedModel != null) {
              setState(() {
                selectedSpaceModel = selectedModel;
                subspaces = null;
              });
            }
          },
        );
      },
    );
  }

  void _showSubSpaceDialog(
      BuildContext context,
      String name,
      final List<Tag>? spaceTags,
      bool isEdit,
      List<ProductModel>? products,
      final List<SubspaceModel>? existingSubSpaces) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return CreateSubSpaceDialog(
            spaceName: name,
            dialogTitle: 'Create Sub-space',
            spaceTags: spaceTags,
            isEdit: isEdit,
            products: products,
            existingSubSpaces: existingSubSpaces,
            onSave: (slectedSubspaces) {
              if (slectedSubspaces != null) {
                setState(() {
                  subspaces = slectedSubspaces;
                  selectedSpaceModel = null;
                });
              }
            });
      },
    );
  }

  void _showTagCreateDialog(
      BuildContext context, String name, List<ProductModel>? products) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AddDeviceTypeWidget(
          spaceName: name,
          products: products,
          subspaces: subspaces,
          spaceTags: tags,
          allTags: [],
          initialSelectedProducts:
              createInitialSelectedProducts(tags, subspaces),
          onSave: (selectedSpaceTags, selectedSubspaces) {
            setState(() {
              tags = selectedSpaceTags;
              selectedSpaceModel = null;

              if (selectedSubspaces != null) {
                if (subspaces != null) {
                  for (final subspace in subspaces!) {
                    for (final selectedSubspace in selectedSubspaces) {
                      if (subspace.subspaceName ==
                          selectedSubspace.subspaceName) {
                        subspace.tags = selectedSubspace.tags;
                      }
                    }
                  }
                }
              }
            });
          },
        );
      },
    );
  }

  List<SelectedProduct> createInitialSelectedProducts(
      List<Tag>? tags, List<SubspaceModel>? subspaces) {
    final Map<ProductModel, int> productCounts = {};

    if (tags != null) {
      for (var tag in tags) {
        if (tag.product != null) {
          productCounts[tag.product!] = (productCounts[tag.product!] ?? 0) + 1;
        }
      }
    }

    if (subspaces != null) {
      for (var subspace in subspaces) {
        if (subspace.tags != null) {
          for (var tag in subspace.tags!) {
            if (tag.product != null) {
              productCounts[tag.product!] =
                  (productCounts[tag.product!] ?? 0) + 1;
            }
          }
        }
      }
    }

    return productCounts.entries
        .map((entry) => SelectedProduct(
              productId: entry.key.uuid,
              count: entry.value,
              productName: entry.key.name ?? 'Unnamed',
              product: entry.key,
            ))
        .toList();
  }

  Map<ProductModel, int> _groupTags(List<Tag> tags) {
    final Map<ProductModel, int> groupedTags = {};
    for (var tag in tags) {
      if (tag.product != null) {
        groupedTags[tag.product!] = (groupedTags[tag.product!] ?? 0) + 1;
      }
    }
    return groupedTags;
  }
}
