import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/bloc/subspace_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/bloc/subspace_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/utils/constants/action_enum.dart';

class SubSpaceModelBloc extends Bloc<SubSpaceModelEvent, SubSpaceModelState> {
  SubSpaceModelBloc() : super(SubSpaceModelState([], [], '', {})) {
    // Handle AddSubSpaceModel Event
    on<AddSubSpaceModel>((event, emit) {
      final existingNames =
          state.subSpaces.map((e) => e.subspaceName.toLowerCase()).toSet();

      if (existingNames.contains(event.subSpace.subspaceName.toLowerCase())) {
        final updatedDuplicates = Set<String>.from(state.duplicates)
          ..add(event.subSpace.subspaceName.toLowerCase());
        final updatedSubSpaces =
            List<SubspaceTemplateModel>.from(state.subSpaces)
              ..add(event.subSpace);
        emit(SubSpaceModelState(
          updatedSubSpaces,
          state.updatedSubSpaceModels,
          '*Duplicated sub-space name',
          updatedDuplicates,
        ));
      } else {
        // Add subspace if no duplicate exists
        final updatedSubSpaces =
            List<SubspaceTemplateModel>.from(state.subSpaces)
              ..add(event.subSpace);

        emit(SubSpaceModelState(
          updatedSubSpaces,
          state.updatedSubSpaceModels,
          '',
          state.duplicates,
// Clear error message
        ));
      }
    });

    // Handle RemoveSubSpaceModel Event
    on<RemoveSubSpaceModel>((event, emit) {
      final updatedSubSpaces = List<SubspaceTemplateModel>.from(state.subSpaces)
        ..remove(event.subSpace);

      final updatedSubspaceModels = List<UpdateSubspaceTemplateModel>.from(
        state.updatedSubSpaceModels,
      );
      final nameOccurrences = <String, int>{};
      for (final subSpace in updatedSubSpaces) {
        final lowerName = subSpace.subspaceName.toLowerCase();
        nameOccurrences[lowerName] = (nameOccurrences[lowerName] ?? 0) + 1;
      }

      final updatedDuplicates = nameOccurrences.entries
          .where((entry) => entry.value > 1)
          .map((entry) => entry.key)
          .toSet();

      if (event.subSpace.uuid?.isNotEmpty ?? false) {
        updatedSubspaceModels.add(UpdateSubspaceTemplateModel(
          action: Action.delete,
          uuid: event.subSpace.uuid!,
        ));
      }

      emit(SubSpaceModelState(
        updatedSubSpaces,
        updatedSubspaceModels,
        '',
        updatedDuplicates,
// Clear error message
      ));
    });

    // Handle UpdateSubSpaceModel Event
    on<UpdateSubSpaceModel>((event, emit) {
      final updatedSubSpaces = state.subSpaces.map((subSpace) {
        if (subSpace.uuid == event.updatedSubSpace.uuid) {
          return event.updatedSubSpace;
        }
        return subSpace;
      }).toList();

      final updatedSubspaceModels = List<UpdateSubspaceTemplateModel>.from(
        state.updatedSubSpaceModels,
      );

      updatedSubspaceModels.add(UpdateSubspaceTemplateModel(
        action: Action.update,
        uuid: event.updatedSubSpace.uuid!,
      ));

      emit(SubSpaceModelState(
        updatedSubSpaces,
        updatedSubspaceModels,
        '',
        state.duplicates,
// Clear error message
      ));
    });
  }
}
