import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/button_content_widget.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/views/create_subspace_model_dialog.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class SubspaceModelCreate extends StatelessWidget {
  final List<SubspaceTemplateModel> subspaces;
  final void Function(List<SubspaceTemplateModel> newSubspaces)?
      onSpaceModelUpdate;

  const SubspaceModelCreate(BuildContext context,
      {Key? key, required this.subspaces, this.onSpaceModelUpdate})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final screenWidth = MediaQuery.of(context).size.width;
    return Container(
      child: subspaces.isEmpty
          ? TextButton(
              style: TextButton.styleFrom(
                overlayColor: ColorsManager.transparentColor,
              ),
              onPressed: () async {
                await _openDialog(context, 'Create Sub-space');
              },
              child: const ButtonContentWidget(
                icon: Icons.add,
                label: 'Create Sub Space',
              ),
            )
          : SizedBox(
              width: screenWidth * 0.25,
              child: Container(
                padding: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  color: ColorsManager.textFieldGreyColor,
                  borderRadius: BorderRadius.circular(15),
                  border: Border.all(
                    color: ColorsManager.textFieldGreyColor,
                    width: 3.0, // Border width
                  ),
                ),
                child: Wrap(
                  spacing: 8.0,
                  runSpacing: 8.0,
                  children: [
                    ...subspaces.map((subspace) => Container(
                          padding: const EdgeInsets.symmetric(
                              horizontal: 8.0, vertical: 4.0),
                          decoration: BoxDecoration(
                            color: ColorsManager.whiteColors,
                            borderRadius: BorderRadius.circular(10),
                            border: Border.all(
                                color: ColorsManager.transparentColor),
                          ),
                          child: Text(
                            subspace.subspaceName,
                            style: Theme.of(context)
                                .textTheme
                                .bodySmall
                                ?.copyWith(color: ColorsManager.spaceColor),
                          ),
                        )),
                    GestureDetector(
                      onTap: () async {
                        await _openDialog(context, 'Edit Sub-space');
                      },
                      child: Chip(
                        label: const Text(
                          'Edit',
                          style: TextStyle(color: ColorsManager.spaceColor),
                        ),
                        backgroundColor: ColorsManager.whiteColors,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(16),
                          side:
                              const BorderSide(color: ColorsManager.spaceColor),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
    );
  }

  Future<void> _openDialog(BuildContext context, String dialogTitle) async {
    await showDialog(
      barrierDismissible: false,
      context: context,
      builder: (BuildContext context) {
        return CreateSubSpaceModelDialog(
          isEdit: true,
          dialogTitle: dialogTitle,
          existingSubSpaces: subspaces,
          onUpdate: (subspaceModels) {
            onSpaceModelUpdate!(subspaceModels);
          },
        );
      },
    );
  }
}
